@@ -37,29 +37,18 @@ exception statement from your version.  */
 
 
 package gnu.javax.crypto.jce.mac;
+
 import gnu.java.security.Registry;
 
 /**
- * <p>The implementation of the HMAC-SHA-512 <i>Service Provider Interface</i>
- * (<b>SPI</b>) adapter.</p>
+ * The implementation of the HMAC-SHA-512 <i>Service Provider Interface</i>
+ * (<b>SPI</b>) adapter.
  */
-public class HMacSHA512Spi extends MacAdapter
+public class HMacSHA512Spi
+    extends MacAdapter
 {
-
-  // Constants and variables
-  // -------------------------------------------------------------------------
-
-  // Constructor(s)
-  // -----------------------------------------------------------------------
-
   public HMacSHA512Spi()
   {
     super(Registry.HMAC_NAME_PREFIX + Registry.SHA512_HASH);
   }
-
-  // Class methods
-  // -------------------------------------------------------------------------
-
-  // Instance methods
-  // -------------------------------------------------------------------------
 }
