@@ -421,7 +421,7 @@ public class SecurityManager
   public void checkAccess(Thread thread)
   {
     if (thread.getThreadGroup() != null 
-	&& thread.getThreadGroup().getParent() == null)
+	&& thread.getThreadGroup().parent == null)
       checkPermission(new RuntimePermission("modifyThread"));
   }
 
@@ -454,7 +454,7 @@ public class SecurityManager
    */
   public void checkAccess(ThreadGroup g)
   {
-    if (g.getParent() == null)
+    if (g.parent == null)
       checkPermission(new RuntimePermission("modifyThreadGroup"));
   }
 
