@@ -1,5 +1,5 @@
 /* JEditorPane.java --
-   Copyright (C) 2002, 2004, 2005  Free Software Foundation, Inc.
+   Copyright (C) 2002, 2004, 2005, 2006,  Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -40,6 +40,8 @@ package javax.swing;
 
 import java.awt.Container;
 import java.awt.Dimension;
+import java.io.BufferedInputStream;
+import java.io.FilterInputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.InputStreamReader;
@@ -47,6 +49,7 @@ import java.io.Reader;
 import java.io.StringReader;
 import java.net.MalformedURLException;
 import java.net.URL;
+import java.net.URLConnection;
 import java.util.HashMap;
 
 import javax.accessibility.AccessibleContext;
@@ -56,6 +59,8 @@ import javax.accessibility.AccessibleStateSet;
 import javax.accessibility.AccessibleText;
 import javax.swing.event.HyperlinkEvent;
 import javax.swing.event.HyperlinkListener;
+import javax.swing.plaf.TextUI;
+import javax.swing.text.AbstractDocument;
 import javax.swing.text.BadLocationException;
 import javax.swing.text.DefaultEditorKit;
 import javax.swing.text.Document;
@@ -505,9 +510,124 @@ public class JEditorPane extends JTextComponent
     }
   }
 
+  /**
+   * A special stream that can be cancelled.
+   */
+  private class PageStream
+    extends FilterInputStream
+  {
+    /**
+     * True when the stream has been cancelled, false otherwise.
+     */
+    private boolean cancelled;
+
+    protected PageStream(InputStream in)
+    {
+      super(in);
+      cancelled = false;
+    }
+
+    private void checkCancelled()
+      throws IOException
+    {
+      if (cancelled)
+        throw new IOException("Stream has been cancelled");
+    }
+
+    void cancel()
+    {
+      cancelled = true;
+    }
+
+    public int read()
+      throws IOException
+    {
+      checkCancelled();
+      return super.read();
+    }
+
+    public int read(byte[] b, int off, int len)
+      throws IOException
+    {
+      checkCancelled();
+      return super.read(b, off, len);
+    }
+
+    public long skip(long n)
+      throws IOException
+    {
+      checkCancelled();
+      return super.skip(n);
+    }
+
+    public int available()
+      throws IOException
+    {
+      checkCancelled();
+      return super.available();
+    }
+
+    public void reset()
+      throws IOException
+    {
+      checkCancelled();
+      super.reset();
+    }
+  }
+
+  /**
+   * The thread that loads documents asynchronously.
+   */
+  private class PageLoader
+    implements Runnable
+  {
+    private Document doc;
+    private PageStream in;
+    private URL old;
+    URL page;
+    PageLoader(Document doc, InputStream in, URL old, URL page)
+    {
+      this.doc = doc;
+      this.in = new PageStream(in);
+      this.old = old;
+      this.page = page;
+    }
+
+    public void run()
+    {
+      try
+        {
+          read(in, doc);
+        }
+      catch (IOException ex)
+        {
+          UIManager.getLookAndFeel().provideErrorFeedback(JEditorPane.this);
+        }
+      finally
+        {
+          if (SwingUtilities.isEventDispatchThread())
+            firePropertyChange("page", old, page);
+          else
+            {
+              SwingUtilities.invokeLater(new Runnable()
+              {
+                public void run()
+                {
+                  firePropertyChange("page", old, page);
+                }
+              });
+            }
+         }
+     }
+
+     void cancel()
+     {
+       in.cancel();
+     }
+  }
+
   private static final long serialVersionUID = 3140472492599046285L;
   
-  private URL page;
   private EditorKit editorKit;
   
   boolean focus_root;
@@ -518,6 +638,11 @@ public class JEditorPane extends JTextComponent
   // A mapping between content types and used EditorKits
   HashMap editorMap;  
 
+  /**
+   * The currently loading stream, if any.
+   */
+  private PageLoader loader;
+
   public JEditorPane()
   {
     init();
@@ -538,8 +663,8 @@ public class JEditorPane extends JTextComponent
 
   public JEditorPane(URL url) throws IOException
   {
-    init ();
-    setEditorKit (createEditorKitForContentType("text/html"));;
+    init();
+    setEditorKit(createEditorKitForContentType("text/html"));;
     setPage(url);
   }
   
@@ -581,12 +706,15 @@ public class JEditorPane extends JTextComponent
     // TODO: Have to handle the case where a ClassLoader was specified
     // when the EditorKit was registered
     EditorKit e = null;
-    String className = (String)registerMap.get(type);
+    String className = (String) registerMap.get(type);
     if (className != null)
       {
         try
         {
-          e = (EditorKit) Class.forName(className).newInstance();
+          // XXX - This should actually depend on the classloader
+          // registered with the type. See registerEditorKitForContentType.
+          ClassLoader ldr = ClassLoader.getSystemClassLoader();
+          e = (EditorKit) Class.forName(className, true, ldr).newInstance();
         }
         catch (Exception e2)
         {    
@@ -695,10 +823,28 @@ public class JEditorPane extends JTextComponent
   public Dimension getPreferredSize()
   {
     Dimension pref = super.getPreferredSize();
-    if (getScrollableTracksViewportWidth())
-      pref.width = getUI().getMinimumSize(this).width;
-    if (getScrollableTracksViewportHeight())
-      pref.height = getUI().getMinimumSize(this).height;
+    Container parent = getParent();
+    if (parent instanceof JViewport)
+      {
+        JViewport vp = (JViewport) getParent();
+        TextUI ui = getUI();
+        Dimension min = null;
+        if (! getScrollableTracksViewportWidth())
+          {
+            min = ui.getMinimumSize(this);
+            int vpWidth = vp.getWidth();
+            if (vpWidth != 0 && vpWidth < min.width)
+              pref.width = min.width;
+          }
+        if (! getScrollableTracksViewportHeight())
+          {
+            if (min == null)
+              min = ui.getMinimumSize(this);
+            int vpHeight = vp.getHeight();
+            if (vpHeight != 0 && vpHeight < min.height)
+              pref.height = min.height;
+          }
+      }
     return pref;
   }
 
@@ -716,8 +862,11 @@ public class JEditorPane extends JTextComponent
     // Tests show that this returns true when the parent is a JViewport
     // and has a height > minimum UI height.
     Container parent = getParent();
+    int height = parent.getHeight();
+    TextUI ui = getUI();
     return parent instanceof JViewport
-           && parent.getHeight() > getUI().getMinimumSize(this).height;
+           && height >= ui.getMinimumSize(this).height
+           && height <= ui.getMaximumSize(this).height;
   }
 
   /**
@@ -740,13 +889,19 @@ public class JEditorPane extends JTextComponent
 
   public URL getPage()
   {
-    return page;
+    return loader != null ? loader.page : null;
   }
 
   protected InputStream getStream(URL page)
     throws IOException
   {
-    return page.openStream();
+    URLConnection conn = page.openConnection();
+    // Try to detect the content type of the stream data.
+    String type = conn.getContentType();
+    if (type != null)
+      setContentType(type);
+    InputStream stream = conn.getInputStream();
+    return new BufferedInputStream(stream);
   }
 
   public String getText()
@@ -777,10 +932,12 @@ public class JEditorPane extends JTextComponent
     EditorKit kit = getEditorKit();
     if (kit instanceof HTMLEditorKit && desc instanceof HTMLDocument)
       {
-        Document doc = (Document) desc;
+        HTMLDocument doc = (HTMLDocument) desc;
+        setDocument(doc);
         try
           {
-            kit.read(in, doc, 0);
+            InputStreamReader reader = new InputStreamReader(in);
+            kit.read(reader, doc, 0);
           }
         catch (BadLocationException ex)
           {
@@ -839,6 +996,13 @@ public class JEditorPane extends JTextComponent
 
   public final void setContentType(String type)
   {
+    // Strip off content type parameters.
+    int paramIndex = type.indexOf(';');
+    if (paramIndex > -1)
+      {
+        // TODO: Handle character encoding.
+        type = type.substring(0, paramIndex).trim();
+      }
     if (editorKit != null
 	&& editorKit.getContentType().equals(type))
       return;
@@ -899,14 +1063,45 @@ public class JEditorPane extends JTextComponent
     if (page == null)
       throw new IOException("invalid url");
 
-    try
+    URL old = getPage();
+    // Only reload if the URL doesn't point to the same file.
+    // This is not the same as equals because there might be different
+    // URLs on the same file with different anchors.
+    if (old == null || ! old.sameFile(page))
       {
-	this.page = page;
-	getEditorKit().read(page.openStream(), getDocument(), 0);
-      }
-    catch (BadLocationException e)
-      {
-	// Ignored. '0' is always a valid offset.
+        InputStream in = getStream(page);
+        if (editorKit != null)
+          {
+            Document doc = editorKit.createDefaultDocument();
+            doc.putProperty(Document.StreamDescriptionProperty, page);
+
+            if (loader != null)
+              loader.cancel();
+            loader = new PageLoader(doc, in, old, page);
+
+            int prio = -1;
+            if (doc instanceof AbstractDocument)
+              {
+                AbstractDocument aDoc = (AbstractDocument) doc;
+                prio = aDoc.getAsynchronousLoadPriority();
+              }
+            if (prio >= 0)
+              {
+                // Load asynchronously.
+                setDocument(doc);
+                Thread loadThread = new Thread(loader,
+                                               "JEditorPane.PageLoader");
+                loadThread.setDaemon(true);
+                loadThread.setPriority(prio);
+                loadThread.start();
+              }
+            else
+              {
+                // Load synchronously.
+                loader.run();
+                setDocument(doc);
+              }
+          }
       }
   }
 
